import java.io.*;
import java.util.Hashtable;
import java.util.Vector;

import java.beans.BeanInfo;
import java.beans.Introspector;
import java.beans.IntrospectionException;
import java.beans.PropertyDescriptor;

import org.apache.axis.message.SOAPHandler;

import org.apache.axis.utils.QName;
import org.apache.axis.utils.JavaUtils;
import org.apache.log4j.Category;

import org.xml.sax.*;

import org.apache.axis.encoding.*;

/**
 * General purpose serializer/deserializerFactory for an arbitrary java bean.
 *
 * @author Sam Ruby <rubys@us.ibm.com>
 */
public class OrderSerializer extends BeanSerializer
{
    /**
     * An array of nothing, defined only once.
     */
    private static final Object[] noArgs = new Object[] {};







    /**
     * Deserializer interface called on each child element encountered in
     * the XML stream.
     */
    public SOAPHandler onStartChild(String namespace,
                                    String localName,
                                    String prefix,
                                    Attributes attributes,
                                    DeserializationContext context)
        throws SAXException
    {
        if(isTargetedProperty(localName))
            return super.onStartChild(namespace, localName, prefix, attributes, context);
        return null;
    }

    /**
     * Serialize a bean.  Done simply by serializing each bean property.
     */
    public void serialize(QName name, Attributes attributes,
                          Object value, SerializationContext context)
        throws IOException
    {
        context.startElement(name, attributes);
        PropertyDescriptor[] pd = getPd(value);

        try {
            for (int i=0; i<pd.length; i++) {
                String propName = pd[i].getName();
                if (!isTargetedProperty(propName)) continue;
                if (propName.equals("class")) continue;
                Object propValue = pd[i].getReadMethod().invoke(value,noArgs);
                context.serialize(new QName("", propName), null, propValue);
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new IOException(e.toString());
        }

        context.endElement();
    }
    /**
     * Default constructor.
     */
    public OrderSerializer() {
        super();
    }

    /**
     * Constructor that takes a class.  Provided only for convenience.
     * Equivalent to calling setCls(cls) on a new instance.
     */
    public OrderSerializer(Class cls) {
        super(cls);
    }

    private java.lang.String[] targetProps = {"orderId", "billTo", "shipTo", "sku", "productName", "quantity", "totalPrice"};

/**
 * @return boolean
 */
    protected boolean isTargetedProperty(String prop) {
        for(int i=0; i<targetProps.length; i++)
            if(prop.equals(targetProps[i])) return true;
        return false;
    }
}
