import java.rmi.RemoteException;
import java.util.Properties;
import javax.ejb.*;
import javax.naming.InitialContext;
import java.rmi.RemoteException;

public class POProcessBean implements SessionBean {
    private javax.ejb.SessionContext mySessionContext = null;

    static int MAX_TOTAL=200;
    private ProductHome productHome = null;
    private OrderHome orderHome = null;
    
    public void ejbActivate() throws java.rmi.RemoteException {}
    public void ejbCreate() throws javax.ejb.CreateException, java.rmi.RemoteException {
        try {
            findHomes();
        }
        catch (Exception e) {
            throw new RemoteException(e.toString());
        }
    }
    public void findHomes() throws javax.naming.NamingException {
        try {
            InitialContext initialContext = new InitialContext();
            this.productHome = 
                (ProductHome) javax.rmi.PortableRemoteObject.narrow(
                    initialContext.lookup("Product"), 
                    ProductHome.class);
            this.orderHome = 
                (OrderHome) javax.rmi.PortableRemoteObject.narrow(
                    initialContext.lookup("Order"), 
                    OrderHome.class);
        } catch (javax.naming.NamingException  e) {
            e.printStackTrace();
            throw e;
        }
    }

    public void ejbPassivate() throws java.rmi.RemoteException {}
    public void ejbRemove() throws java.rmi.RemoteException {}
    public javax.ejb.SessionContext getSessionContext() {
        return mySessionContext;
    }
    public void setSessionContext(javax.ejb.SessionContext ctx) throws java.rmi.RemoteException {
        mySessionContext = ctx;
    }

    public void initialize() throws java.rmi.RemoteException
    {
        String sku = "284-ST";
        String name = "STooter";
        String type = "scooter";
        String desc = "Single user scooter - only in red";
        int price = 72;
        int inStock = 50;
        Product product = null;
        try {
            product  = productHome.findByPrimaryKey(sku);
            System.out.println("product found");
        } catch(FinderException e) {
            System.out.println("product not found");
        }               
        try {
            if (product==null)
                product = productHome.create(sku);
            product.setName(name);
            product.setType(type);
            product.setDesc(desc);
            product.setPrice(price);
            product.setInStock(inStock);
            System.out.println("sku created");
        } catch(CreateException e) {
            e.printStackTrace();
            throw new EJBException("Fail in Order.init: " +e.getMessage());
        }
    }

    public Order order(String shipId, String billId, String sku, int quantity) 
        throws POProcessException, java.rmi.RemoteException
    {
        try {
            Product product  = productHome.findByPrimaryKey(sku);
            if (quantity>product.getInStock()) {
                throw( new POProcessException("Stock is not enough"));
            }
            product.setInStock(product.getInStock()-quantity);            
            Order order  = orderHome.create(""+System.currentTimeMillis());
            System.out.println("order class: " + order.getClass() );
            order.setBillTo(billId);
            order.setShipTo(shipId);
            order.setSKU(sku);
            order.setProductName(product.getName());
            order.setQuantity(quantity);
            int total=quantity*product.getPrice();
            order.setTotalPrice(total);
            if (total>MAX_TOTAL) {
                throw( new POProcessException("Exceed the max charge ("+MAX_TOTAL+")"));
            }
            return order;
        } catch(POProcessException e) {
            mySessionContext.setRollbackOnly();
            throw e;
        } catch(CreateException e) {
            throw new EJBException("Fail in Order.order: " +e.getMessage());
        } catch(FinderException e) {
            throw new EJBException("Fail in Order.order: " +e.getMessage());
        }
    }
}
