import java.rmi.RemoteException;
import java.util.Properties;
import javax.ejb.*;
import javax.jms.*;
import javax.naming.InitialContext;
import java.rmi.RemoteException;

public class POProcessBeanJMS implements SessionBean {
    private javax.ejb.SessionContext mySessionContext = null;

    static int MAX_TOTAL=200;
    private ProductHome productHome = null;
    private OrderHome orderHome = null;
    
    public void ejbActivate() throws java.rmi.RemoteException {}
    public void ejbCreate() throws javax.ejb.CreateException, java.rmi.RemoteException {
        try {
            findHomes();
        }
        catch (Exception e) {
            throw new RemoteException(e.toString());
        }
    }
    public void findHomes() throws javax.naming.NamingException {
        try {
            InitialContext initialContext = new InitialContext();
            this.productHome = 
                (ProductHome) javax.rmi.PortableRemoteObject.narrow(
                    initialContext.lookup("Product"), 
                    ProductHome.class);
            this.orderHome = 
                (OrderHome) javax.rmi.PortableRemoteObject.narrow(
                    initialContext.lookup("Order"), 
                    OrderHome.class);
        } catch (javax.naming.NamingException  e) {
            e.printStackTrace();
            throw e;
        }
    }

    public void ejbPassivate() throws java.rmi.RemoteException {}
    public void ejbRemove() throws java.rmi.RemoteException {}
    public javax.ejb.SessionContext getSessionContext() {
        return mySessionContext;
    }
    public void setSessionContext(javax.ejb.SessionContext ctx) throws java.rmi.RemoteException {
        mySessionContext = ctx;
    }

    public void initialize() throws java.rmi.RemoteException, Exception
    {
        try {
            String sku = "284-ST";
            String name = "STooter";
            String type = "scooter";
            String desc = "Single user scooter - only in red";
            int price = 72;
            int inStock = 50;
            Product product = productHome.create(sku);
            product.setName(name);
            product.setType(type);
            product.setDesc(desc);
            product.setPrice(price);
            product.setInStock(inStock);
        } catch(Exception e) {
            e.printStackTrace(); 
            throw e;
        }
    }

    public OrderData order(String shipId, String billId, String sku, int quantity) 
        throws POProcessException
    {
        try {
            Product product = productHome.findByPrimaryKey(sku);
            if (quantity > product.getInStock()) {
                throw new POProcessException("Stock is not enough");
            }
            product.setInStock(product.getInStock() - quantity);            
            OrderData order = new OrderData("" + System.currentTimeMillis());
            order.setBillTo(billId);
            order.setShipTo(shipId);
            order.setSKU(sku);
            order.setProductName(product.getName());
            order.setQuantity(quantity);
            int total = quantity * product.getPrice();
            order.setTotalPrice(total);
            queueConnectionFactory = (QueueConnectionFactory)
                jndiContext.lookup("QueueConnectionFactory");
            queue = (Queue)jndiContext.lookup(queueName); 
            queueConnection =
                queueConnectionFactory.createQueueConnection();
            queueSession =
                queueConnection.createQueueSession(false, 
                    Session.AUTO_ACKNOWLEDGE);
            queueSender = queueSession.createSender(queue);
            ObjectMessage message = queueSession.createObjectMessage();
            message.setObject(order);
            queueSender.send(message);
            if (total > MAX_TOTAL) {
                throw new POProcessException("Exceed the max charge (" + 
                              MAX_TOTAL + ")");
            }
            return order;
        } catch(POProcessException e) {
            mySessionContext.setRollbackOnly();
            throw e;
        } catch(RemoteException e) {
            throw new EJBException("Fail in Order.order: " +e.getMessage());
        } catch(Exception e) {
            throw new EJBException("Fail in Order.order: " +e.getMessage());
        }
    }
    QueueConnectionFactory queueConnectionFactory;
    Queue queue;
    QueueConnection queueConnection;
    QueueSession queueSession;
    QueueSender queueSender;
    String queueName;
    InitialContext jndiContext;
}
