import java.io.*;
import java.net.*;
import java.util.*;

import org.apache.axis.AxisFault;
import org.apache.axis.Constants;
import org.apache.axis.client.ServiceClient;
import org.apache.axis.client.Transport;
import org.apache.axis.transport.http.HTTPTransport;
import org.apache.axis.utils.Options;
import org.apache.axis.utils.QName;
import org.apache.axis.encoding.ServiceDescription;
import org.apache.axis.encoding.SOAPTypeMappingRegistry;
import org.apache.axis.encoding.BeanSerializer;

public class POProcessCaller {
    private static final String SERVICE_NAME = "urn:PurchaseOrder";

    final String targetURL;
    public POProcessCaller(String targetURL) {
        this.targetURL = targetURL;
    }

    public void initialize() throws AxisFault {
        ServiceClient client = new ServiceClient(targetURL);
        client.invoke(SERVICE_NAME, "initialize", new Object[0]);
    }

    public OrderObject order(String shipId,
                             String billId,
                             String sku,
                             int quantity)
        throws AxisFault
    {
        ServiceClient client = new ServiceClient(targetURL);
        QName qn1 = new QName("urn:xml-soap-poejb-demo", "order");
        Class cls = OrderObject.class;
        client.addSerializer(cls, qn1, new OrderSerializer(cls));
        client.addDeserializerFactory(qn1, cls, new OrderDeserializerFactory());

        Object params[] =
            new Object[]{ shipId, billId, sku, new Integer(quantity)};
        Object resp = client.invoke(SERVICE_NAME, "order", params);
        return (OrderObject)resp;
    }

    public static void main (String[] args) throws Exception {
        // args[0]: url
        // args[1]: init/order
        Options opts = new Options( args );
        args = opts.getRemainingArgs();

        if (args.length != 2) {
            System.err.println ("Usage: java " + POProcessCaller.class.getName () + " SOAP-router-URL" + " init/order");
            System.exit (1);
        }

        POProcessCaller caller = new POProcessCaller(args[0]);
        if (args[1].equals("init"))
            caller.initialize();
        else if (args[1].equals("order"))
            caller.order("SkateboardWarehouse", "SkateboardWarehouse", "284-ST", 3);
    }
}
