package ch5.ex6;

import java.io.FileInputStream;
import java.io.InputStream;
import java.io.StringWriter;
import org.apache.axis.client.AxisClient;
import org.apache.axis.client.Transport;
import org.apache.axis.encoding.SerializationContext;
import org.apache.axis.configuration.FileProvider;
import org.apache.axis.message.SOAPEnvelope;
import org.apache.axis.message.SOAPBodyElement;
import org.apache.axis.client.ServiceClient;
import org.apache.axis.Message;
import org.apache.axis.MessageContext;
import ch5.ex6.POSubmissionClient;
import ch5.jms.JMSTransport;

/**
 * Purchase order submission client
 */
public class POSubmissionClient {
    /**
     * Target service URL
     */
    final String queue;
    final String factory = "QueueConnectionFactory";
    final String configFile;
    
    public static void main(String[] args) throws Exception {
        System.out.println(System.getProperties());
        POSubmissionClient client = new POSubmissionClient(args[0], args[1]);
        String resp = client.invoke(new FileInputStream(args[2]));
        System.out.println("Response: ");
        System.out.println(resp);
    }

    /**
     * Create a client with a target URL
     */
    public POSubmissionClient(String queue, String configFile) {
        this.queue = queue;
        this.configFile = configFile;
    }
    
    /**
     * Invoke the PO submission web service
     *
     * @param po Purchase order document
     * @return Invoice document
     * @exception Exception I/O error or Axis error
     */
    public String invoke(InputStream po) throws Exception {
        // Send the message
        Transport transport = 
            new JMSTransport(this.factory, this.queue);

        FileProvider provider =  new FileProvider(configFile);
        AxisClient engine = new AxisClient(provider);

        ServiceClient client = new ServiceClient(transport, engine);
        client.setRequestMessage(new Message(po, true));
        client.getMessageContext().
               setTargetService("http://www.skatestown.com/ns/po");
        client.invoke();
        
        // Retrieve the response body
        MessageContext ctx = client.getMessageContext();
        Message outMsg = ctx.getResponseMessage();
        SOAPEnvelope envelope = outMsg.getAsSOAPEnvelope();
        SOAPBodyElement body = envelope.getFirstBody();
        
        // Get the XML from the body
        StringWriter w = new StringWriter();
        SerializationContext sc = new SerializationContext(w, ctx);
        body.output(sc);
        return w.toString();
    }
}
