package ch5.jms;

import java.io.*;
import java.util.Properties;
import javax.jms.*;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import org.apache.axis.AxisEngine;
import org.apache.axis.AxisFault;
import org.apache.axis.Message;
import org.apache.axis.MessageContext;
import org.apache.axis.configuration.FileProvider;
import org.apache.axis.server.AxisServer;
import org.apache.axis.message.SOAPEnvelope;
import org.apache.axis.message.SOAPFaultElement;
import org.apache.log4j.Category;

public class JMSListener implements MessageListener {
    private AxisEngine engine=null;
    private String factoryName;
    private String queueName;
    private String icf = null;
    private String providerURL = null;
    private String transportName;
    QueueConnectionFactory factory = null;
    QueueConnection connection = null;
    QueueSession session = null;
    QueueReceiver receiver = null;
    QueueSender sender = null;
    Queue queue = null, replyQueue = null;
    InitialContext jndiContext = null;
    javax.jms.Message msg;
    static Category category =
            Category.getInstance(JMSListener.class.getName());

    public JMSListener() {
        this.transportName = "jms";
    }

    public void setFactory(String factoryName) {
        this.factoryName = factoryName;
    }

    public String getFactory() {
        return factoryName;
    }

    public void setQueue(String queueName) {
        this.queueName = queueName;
    }

    public String getQueue() {
        return queueName;
    }

    public void setContextFactory(String icf) {
        this.icf = icf;
    }

    public String getContextFactory() {
        return icf;
    }

    public void setProviderURL(String providerURL) {
        this.providerURL = providerURL;
    }

    public String getProviderURL() {
        return providerURL;
    }

    public void start() {
        try {
            Properties properties = new Properties();
            if (providerURL != null)
                properties.put(javax.naming.Context.PROVIDER_URL, providerURL);
            if (icf != null)
                properties.put(javax.naming.Context.INITIAL_CONTEXT_FACTORY, icf);
            jndiContext = new InitialContext(properties);
            /* Get factory and queue objects by searching via jndi */
            factory = (QueueConnectionFactory) jndiContext.lookup("QueueConnectionFactory");
            queue = (Queue) jndiContext.lookup(this.queueName);
        } catch (NamingException e) {
            System.err.println("Error at JMSProcessor(): " + e);
            System.exit(1);
        }
        try {
            connection = factory.createQueueConnection();
            /* Create a session that automatically acknowledges message receipt */
            session = connection.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);
            receiver = session.createReceiver(queue);
            connection.start(); //start the connection
            System.out.println("Ready to receive messages!!\n");
            /* Let this JMSListener be notified everytime a message is received
            by its associated queue. */
            receiver.setMessageListener(this);
            System.out.println("\nTo stop receiving messages, type 'stop'.\n");
            String input = "";
            /* Wait until the server administrator stops the service. */
            BufferedReader reader = new BufferedReader(new InputStreamReader(System.in));
            while (!(input.toLowerCase()).equals("stop")) {
                try {
                    input = reader.readLine();
                } catch (IOException e) {
                    System.err.println("IOException while reading from Stdin\n");
                }
            }
            if (connection != null) {
                connection.close();
            }
        } catch (JMSException e) {
            if (connection != null) {
                try {
                    connection.close();
                } catch (JMSException e2) {
                    System.err.println("Exception at JMSListener.start: " + e2);
                    System.exit(1);
                }
            }
            System.err.println("Exception at JMSListener.start: " + e);
            System.exit(1);
        }
    }
    public AxisEngine getAxisEngine() throws Exception {
        if (engine != null)
            return engine;
        String config = "server-config.xml";
        String value;
        if ((value = System.getProperty("ch5.jms.config")) != null)
            config = value;
        System.out.println("config file: " + config);

        FileProvider provider =  new FileProvider(config);
        return new AxisServer(provider);
    }
    public void onMessage(javax.jms.Message msg) {
        BytesMessage bytesMsg = null;
        org.apache.axis.Message respMessage = null;
        MessageContext msgContext = null;
        String msgTxt = null, respString = "";

        try {
            AxisEngine engine = getAxisEngine();
            msgContext = new MessageContext(engine);
            if (msg instanceof BytesMessage) {
                bytesMsg = (BytesMessage) msg;
                /* Get the queue that the reply should be directed to. */
                replyQueue = (Queue)(bytesMsg.getJMSReplyTo());
                sender = session.createSender(replyQueue);
                msgTxt = bytesMsg.readUTF();
                org.apache.axis.Message soapMessage = new org.apache.axis.Message(msgTxt);
                msgContext.setRequestMessage(soapMessage);
                msgContext.setTransportName(transportName);
                /* Let the engine take care of processing the SOAP request. */
                engine.invoke(msgContext);
            } else {
                /* If the message received by the queue is not a BytesMessage,
                then throw a new Axis Fault. */
               throw new AxisFault("AxisServer.error", "Wrong message type.", null, null);
            }

        } catch (Exception e) {
            e.printStackTrace();
            if ( !(e instanceof AxisFault) )
                e = new AxisFault( e );
            respMessage = msgContext.getResponseMessage();
            if (respMessage == null) {
                respMessage = new org.apache.axis.Message((AxisFault)e);
                msgContext.setResponseMessage(respMessage);
            } else {
                try {
                    SOAPEnvelope env = respMessage.getAsSOAPEnvelope();
                    env.clearBody();
                    env.addBodyElement(new SOAPFaultElement((AxisFault)e));
                } catch (AxisFault af) {
                    af.printStackTrace();
                }
            }
        }

        try {
            respMessage = msgContext.getResponseMessage();
            if (respMessage != null)
                respString = respMessage.getAsString();
            BytesMessage reply = session.createBytesMessage();
            reply.writeUTF(respString);
            /* Send the response back to JMSSender via JMS. */
            sender.send(reply);
        } catch (Exception e) {
            System.err.println("Exception at JMSListener.onMessage: " + e);
        }
        System.out.println("\nTo stop receiving messages, type 'stop'.\n");
    }


    public static void main(String args[]) {
        String fName, qName, icfName = null, providerURLName = null;

        if (args.length < 2) {
            fName = "JMSTestFactory"; // default Factory name
            qName = "MyQueue"; // default Queue name
        } else {
            fName = args[0];
            qName = args[1];
            if (args.length > 2) icfName = args[2];
            if (args.length > 3) providerURLName = args[3];
        }
        JMSListener listener = new JMSListener();
        listener.setQueue(qName);
        listener.setFactory(fName);
        if (icfName != null) listener.setContextFactory(icfName);
        if (providerURLName != null) listener.setProviderURL(providerURLName);
        listener.start();
    }
}
