package ch5.jms;

import java.util.Properties;
import javax.jms.*;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import org.apache.axis.AxisFault;
import org.apache.axis.Message;
import org.apache.axis.MessageContext;
import org.apache.axis.handlers.BasicHandler;
import org.apache.log4j.Category;

public class JMSSender extends BasicHandler {
    private String queueName;
    private String factoryName;
    private String messageMode;
    private String icf;
    private String providerURL;
    private InitialContext jndiContext = null;
    private QueueConnectionFactory queueFactory = null;
    private QueueConnection queueConnection = null;
    private QueueSession queueSession = null;
    private QueueSender queueSender = null;
    private Queue queue = null;

    public void invoke(MessageContext msgContext) throws AxisFault {
        /* Get the factory and name of the delivery medium. */
        factoryName = (String)msgContext.getProperty(JMSConstants.MC_JMS_FACTORY_NAME);
        queueName = (String)msgContext.getProperty(JMSConstants.MC_JMS_QUEUE_NAME);
        icf = (String)msgContext.getProperty(JMSConstants.MC_JMS_INITIAL_CONTEXT_FACTORY);
        providerURL = (String)msgContext.getProperty(JMSConstants.MC_JMS_PROVIDER_URL);
        /* Throw a fault is any of the three is not defined. */
        if ((factoryName == null)  || (queueName == null)) {
            throw new AxisFault("JMSSender.invoke", "Missing mode, factory, or queue specifier.", null, null);
        }
        try {
            Properties properties = new Properties();
            if (providerURL != null)
                properties.put(javax.naming.Context.PROVIDER_URL, providerURL);
            if (icf != null)
                properties.put(javax.naming.Context.INITIAL_CONTEXT_FACTORY, icf);
            jndiContext = new javax.naming.InitialContext(properties);
        } catch (NamingException e) {
            throw new AxisFault("JMSSender.invoke", "Context naming exception.", null, null);
        }
        getJMSObjects();
        org.apache.axis.Message message = msgContext.getRequestMessage();
        String messageTxt = message.getAsString();
        sendMessage(msgContext, messageTxt);
    }

    private void sendMessage(MessageContext msgContext, String messageTxt) throws AxisFault {
        BytesMessage msg;

        try {
            queueConnection = queueFactory.createQueueConnection();
            /* Create a session with auto message receipt acknowledgement. */
            queueSession = queueConnection.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);
            queueSender = queueSession.createSender(queue);
            msg = queueSession.createBytesMessage();
            msg.writeUTF(messageTxt);
            /* Create a temporary queue to hold the reply. */
            TemporaryQueue replyTo = queueSession.createTemporaryQueue();
            msg.setJMSReplyTo(replyTo); // set the reply queue to the temporary queue.
            queueSender.send(msg);
            queueConnection.start();
            QueueReceiver receiver = queueSession.createReceiver(replyTo);
            /* Block until a reply is received. */
            javax.jms.Message replyMsg = receiver.receive();
            if (replyMsg instanceof BytesMessage) {
                String replyTxt = ((BytesMessage)replyMsg).readUTF();
                org.apache.axis.Message respMsg = new org.apache.axis.Message(replyTxt);
                msgContext.setResponseMessage(respMsg);
            }
        } catch (JMSException e) {
            throw new AxisFault("JMSSender.sendMessage", "JMSException: " + e, null, null);
        } finally {
            try {
                if (queueConnection != null) queueConnection.close();
            } catch (JMSException e) {}
        }
    }

    private void getJMSObjects() throws AxisFault {
        /* Get the queue and factory objects according to the mode of the medium. */
        try {
            queueFactory = (QueueConnectionFactory)jndiContext.lookup(factoryName);
            queue =(Queue) jndiContext.lookup(queueName);
        } catch (NamingException e) {
            throw new AxisFault("JMSSender.getJMSObjects", "JMSException: " + e, null, null);
        }
    }

    public void undo(MessageContext msgContext) {
    }

}
